/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.generator.extension.rumblepad;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.platform.raspberry.generator.framework.ISharedInstanceGeneratorExtension;

/**
 * Base class for rumble feature.
 * 
 * @author hoelzl
 */
abstract class RumbleMagnitudeGeneratorExtensionBase<T extends EObject> extends
		RumblepadGeneratorExtensionBase<T> implements ISharedInstanceGeneratorExtension {
	/** Stores the instance postfix. */
	private String sharedInstancePostfix;

	/** Constructor. */
	public RumbleMagnitudeGeneratorExtensionBase(T modelElement) {
		super(modelElement);
	}

	/** {@inheritDoc} */
	@Override
	public final void setSharedInstancePostfix(String instancePostfix) {
		this.sharedInstancePostfix = instancePostfix;
	}

	/** Returns instancePostfix. */
	public final String getSharedInstancePostfix() {
		return sharedInstancePostfix;
	}

	/** {@inheritDoc} */
	@Override
	public String getSharedInstanceIdentifier() {
		return "rumble_feature_" + getHeaderFileName();
	}

	/** {@inheritDoc} */
	@Override
	public String getSharedInstanceVariableDeclarationCode() {
		return "static uint16_t rumble_strong_" + sharedInstancePostfix + " = 0;\n" +
				"static uint16_t rumble_weak_" + sharedInstancePostfix + " = 0;\n";
	}

	/** Returns the variable for the strong rumble feature. */
	protected final String getStrongMagnitudeVariable() {
		return "rumble_strong_" + sharedInstancePostfix;
	}

	/** Returns the variable for the weak rumble feature. */
	protected final String getWeakMagnitudeVariable() {
		return "rumble_weak_" + sharedInstancePostfix;
	}

	/** {@inheritDoc} */
	@Override
	public String getSharedInstanceAuxiliaryFunctions() {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public String getSharedInstanceInitializationCode() {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public String getSharedInstanceTerminationCode() {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public String getSharedInstanceReadCode() {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public String getSharedInstanceWriteCode() {
		return "rumblepad_set_rumble(" + getStrongMagnitudeVariable() + ", " +
				getWeakMagnitudeVariable() + ", cycle_time * 0.9, 0);\n";
	}
}
